#!/bin/sh -e

# Default settings
verfile="tomtom.ver"
defcfg="config-default"
curcfg=".config"
release="n"
rebuild="n"
clean="n"
makeflags="-w"
makedefs="V=0"
makejobs="-j 4"

# Environment for building
export ARCH="arm"
export CROSS_COMPILE="arm-linux-"

die() {
  echo "$0: ERROR: $1"
  exit 1
}

usage() {
  echo "Usage: $0 {release|rebuild|clean|silent|verbose|single}"
  exit 1
}

get_version() {
  dirty=""

  echo "**** Checking if you edited any sources..."
  if ! p4out=$(p4 opened -m 1 ... 2>&1); then
    if test "${release}" = "y"; then
      die "Perforce command \"p4 opened -m 1\" failed: ${p4out}"
    else
      echo "Perforce command \"p4 opened -m 1\" failed: ${p4out}"
    fi
  fi

  if ! echo "${p4out}" | grep -Fq 'file(s) not opened on this client.'; then
    if test "${release}" = "y"; then
      die "You need to commit your changes first."
    else 
      echo; echo "!!! You have unsubmitted changed files. !!!"; echo
      dirty="-dirty"
    fi
  fi

  # always genereate a TTVER based on the last changelist in this dir
  if ! changelist=$(p4 changes -m 1 -s submitted ...#have); then
    if test "${release}" = "y"; then
      die "Perforce command \"p4 changes\" failed: ${changelist}"
    else
      echo "Perforce command \"p4 changes\" failed: ${changelist}"
      changelist=999999
    fi
  else
    changelist=$(echo ${changelist} | awk '{print $2}')
  fi

  #TTVER="${changelist}${dirty}"
  TTVER="${changelist}"
  echo "# Generated version file, do not edit." > ${verfile}
  echo "TTVER=\"${TTVER}\"" >> ${verfile}
  echo "# EOF" >> ${verfile}
}


make_clean() {
  echo "**** Cleaning..."
  nice make ${makeflags} ${makedefs} distclean
}

make_config() {
  echo "**** Configuring with ${defcfg}..."
  cat "${defcfg}" > "${curcfg}"
  nice make ${makeflags} ${makedefs} silentoldconfig
}

make_build() {
  echo "**** Building..."
  nice make ${makeflags} ${makejobs} ${makedefs}
}

# Main starts here
while test -n "$1"; do
  case "$1" in
  release)
    release="y"
    ;;
  rebuild)
    rebuild="y"
    ;;
  clean)
    clean="y"
    ;;
  silent)
    makeflags="-ws"
    makedefs="V=0"
    ;;
  verbose)
    makeflags="-w"
    makedefs="V=1"
    ;;
  single)
    makejobs=""
    ;;
  config-*)
    if test ! -f "$1"; then
      die "$1 does not exist!"
    fi
    if test -f "${curcfg}" && ! rm -f "${curcfg}"; then
      die "Unable to remove ${curcfg}!"
    fi
    defcfg="$1"
    ;;
  *)
    usage
    ;;
  esac
  shift
done

get_version

# Set up our local version suffix
echo "-tt${TTVER}" > localversion

echo "**** Processing kernel #${TTVER}..."

if test "${rebuild}" = "y"; then
  make_clean
  make_config
elif test "${clean}" = "y"; then
  make_clean
  echo "**** Successfully cleaned kernel #${TTVER}."
  exit 0
elif test ! -f "${curcfg}" -o "${defcfg}" -nt "${curcfg}"; then
  make_config
fi

make_build

echo "**** Successfully built kernel #${TTVER}:"
ls -l arch/arm/boot/zImage

# EOF
